# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .log_prob_token import LogProbToken
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class MessageContentLogProb(UncheckedBaseModel):
    token: str
    logprob: float
    bytes: typing.Optional[typing.List[int]] = None
    top_logprobs: typing.Optional[typing.List[LogProbToken]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
