# This file was auto-generated by Fern from our API Definition.

import typing
from ....types.letta_schemas_message_message import LettaSchemasMessageMessage
from .messages_list_response_item import MessagesListResponseItem

MessagesListResponse = typing.Union[typing.List[LettaSchemasMessageMessage], typing.List[MessagesListResponseItem]]
