# This file was auto-generated by Fern from our API Definition.

import typing

ToolRuleType = typing.Union[
    typing.Literal[
        "InitToolRule", "TerminalToolRule", "continue_loop", "conditional", "ToolRule", "require_parent_tools"
    ],
    typing.Any,
]
